package com.fr.plugin.third.party.jsdigei.http;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.data.NetworkHelper;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.mobile.terminal.TerminalHandler;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.login.TokenResource;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.general.ComparatorUtils;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.third.party.jsdigei.Utils;
import com.fr.plugin.third.party.jsdigei.config.CustomDataConfig;
import com.fr.third.org.apache.http.HttpEntity;
import com.fr.third.org.apache.http.HttpHeaders;
import com.fr.third.org.apache.http.HttpStatus;
import com.fr.third.org.apache.http.NameValuePair;
import com.fr.third.org.apache.http.client.config.RequestConfig;
import com.fr.third.org.apache.http.client.entity.UrlEncodedFormEntity;
import com.fr.third.org.apache.http.client.methods.CloseableHttpResponse;
import com.fr.third.org.apache.http.client.methods.HttpGet;
import com.fr.third.org.apache.http.client.methods.HttpPost;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.third.org.apache.http.message.BasicNameValuePair;
import com.fr.third.org.apache.http.util.EntityUtils;
import com.fr.third.springframework.web.util.UriUtils;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


public class SessionGlobalRequestFilterProvider extends AbstractGlobalRequestFilterProvider {
    private static String DEFAULT_USER_AGENT = "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36";

    @Override
    public String filterName() {
        return "com.fr.plugin.third.party.jsdigei";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/decision", "/decision/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {
            if (!PluginContexts.currentContext().isAvailable()) {
                LogKit.info("集成登录,许可证过期");
                filterChain.doFilter(req, res);
                return;
            }

            String reqUrl = req.getRequestURL().toString();
            String fullUrl = Utils.getFullRequestUrl(req);
            String method = req.getMethod();
            LogKit.info("集成登录,记录访问地址：" + method + " " + fullUrl);

            if (!"GET".equalsIgnoreCase(method)) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("/remote/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("/decision/login") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (fullUrl.indexOf("/weixin/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }


            if (fullUrl.indexOf("/dingtalk/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (isAllowIdmOAuthLogin(req)) {
                String state = Utils.getUuid();
                String requestUrl = getRequestUrl(req);
                requestUrl = replaceUrl(requestUrl);
                LogKit.info("集成登录,访问地址," + requestUrl);
                CustomDataConfig.addUrl(state, requestUrl);
                String locationUrl = getOAuthCodeUrl(state, requestUrl);
                Utils.sendRedirect(res, locationUrl);
                return;
            }

            String loginUsername = getIdmOAuthUsername(req);
            if (StringKit.isEmpty(loginUsername)) {
                if (reqUrl.indexOf("/jsdigei-url/") >= 0) {
                    Utils.sendRedirect(res, CustomDataConfig.getInstance().getFrUrl());
                    return;
                }
                filterChain.doFilter(req, res);
                return;
            }

            LogKit.info("集成登录,  OAuth 用户名:" + loginUsername);
            User user = UserService.getInstance().getUserByUserName(loginUsername);
            boolean tipsOption = false;
            String tipsContent = "";
            if (user == null) {
                tipsOption = true;
                LogKit.info("集成登录,用户名:" + loginUsername + "在报表平台不存在");
                tipsContent = "在报表服务器上不存在";
            } else if (!user.isEnable()) {
                tipsOption = true;
                LogKit.info("集成登录,用户名:" + loginUsername + "在报表平台上被禁用");
                tipsContent = "在报表平台上被禁用";
            }

            if (tipsOption) {
                String jumpContent = "<!doctype html>\n" +
                        "<head>\n" +
                        "    <meta charset=\"utf-8\" />\n" +
                        "    <title>提示</title>\n" +
                        "</head>\n" +
                        "<body>\n" +
                        "    <script>\n" +
                        "        var t = 20;\n" +
                        "        var referI =  setInterval(\"refer()\", 1000);\n" +
                        "        function refer() {\n" +
                        "            document.getElementById('show').innerHTML = \"用户:" + loginUsername + tipsContent + ",请联系管理员!<br>\" + t + \"秒后跳转到报表首页\"; \n" +
                        "            t--;\n" +
                        "            if (t <= 0) {\n" +
                        "                clearInterval(referI);\n" +
                        "                window.location = \"" + CustomDataConfig.getInstance().getFrUrl() + "\";\n" +
                        "            }\n" +
                        "        }\n" +
                        "    </script>\n" +
                        "    <div style=\"width: 100%;height:200px;  line-height: 200px;font-size:30px;vertical-align:middle;text-align:center\">\n" +
                        "        <span id=\"show\"></span>\n" +
                        "    </div>\n" +
                        "</body>\n" +
                        "</html>";
                res.setContentType("text/html;charset=UTF-8");
                WebUtils.printAsString(res, jumpContent);
                res.setStatus(200);
                return;
            }

            //loginUsername = user.getUserName();
            //LogKit.info("集成登录,报表平台用户名:" + loginUsername);

            LogKit.info("集成登录,报表平台用户名:" + loginUsername + "生成 login token");
            String loginToken = LoginService.getInstance().login(req, res, loginUsername);
            req.setAttribute("fine_auth_token", loginToken);

            String realUrl = getRealUrl(req);
            if (StringKit.isNotEmpty(realUrl)) {
                LogKit.info("集成登录,真实跳转地址:" + realUrl);
                Utils.sendRedirect(res, realUrl);
                return;
            }

            filterChain.doFilter(req, res);
        } catch (Exception e) {
            LogKit.error("集成登录出错," + e.getMessage(), e);
        }
    }

    private String getRealUrl(HttpServletRequest req) {
        if (req == null) {
            return "";
        }
        String state = getUrlId(req);
        if (StringKit.isEmpty(state)) {
            return "";
        }
        String url = CustomDataConfig.getInstance().getUrlAndDestroy(state);
        return url;
    }

    String getUrlId(HttpServletRequest req) {
        String reqUrl = req.getRequestURL().toString();
        int index = reqUrl.indexOf("/jsdigei-url/");
        if (index < 0) {
            return "";
        }
        int beginIndex = index + 13;
        int endIndex = -1;
        index = reqUrl.indexOf("?", beginIndex);
        if (index > endIndex) {
            endIndex = index;
        }

        index = reqUrl.indexOf("/", beginIndex);
        if (index > endIndex) {
            endIndex = index;
        }
        String urlId = "";

        if (endIndex >= 0) {
            urlId = reqUrl.substring(beginIndex, endIndex);
        } else {
            urlId = reqUrl.substring(beginIndex);
        }
        urlId = StringKit.trim(urlId);
        return urlId;
    }

    public boolean isLogged(HttpServletRequest req) {
        boolean logged = true;

        try {
            String token = TokenResource.COOKIE.getToken(req);
            LoginService.getInstance().loginStatusValid(token, TerminalHandler.getTerminal(req, NetworkHelper.getDevice(req)));
        } catch (Exception var4) {
            logged = false;
        }

        return logged;
    }

    private String getOAuthCodeUrl(String state, String url) throws UnsupportedEncodingException {
        String mappingUrl = getMappingUrl(state);
        String tempUrl = UriUtils.encodeQueryParam(mappingUrl, "UTF-8");
        ;
        LogKit.info("集成登录,授权报表地址:" + url);
        String authUrl = CustomDataConfig.getInstance().getoAuthCodeUrl() + "?client_id=" + CustomDataConfig.getInstance().getIdmClientId() + "&redirect_uri=" + tempUrl + "&response_type=code";
        LogKit.info("集成登录,获取临时授权码地址:" + authUrl);
        return authUrl;
    }

    private String getMappingUrl(String id) {
        String tempUrl = StringKit.trim(CustomDataConfig.getInstance().getFrUrl());
        if (!tempUrl.endsWith("/")) {
            tempUrl = tempUrl + "/";
        }

        tempUrl = tempUrl + "jsdigei-url/" + id;
        return tempUrl;
    }


    private String replaceUrl(String url) {
        if (StringKit.isEmpty(url)) {
            return CustomDataConfig.getInstance().getFrUrl();
        }

        if (url.indexOf("/decision/login") >= 0) {
            return CustomDataConfig.getInstance().getFrUrl();
        }

        String tempUrl = CustomDataConfig.getInstance().getFrUrl();
        int index = tempUrl.indexOf("/decision");
        if (index < 0) {
            return url;
        }

        String pUrl = tempUrl.substring(0, index);

        index = url.indexOf("/decision");
        if (index < 0) {
            return url;
        }
        String fullUrl = pUrl + url.substring(index);
        return fullUrl;
    }


    /**
     * 判断app是否允许登录IDM OAuth
     *
     * @param req
     * @return
     */
    private boolean isAllowIdmOAuthLogin(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        String loginTypeNameParameter = CustomDataConfig.getInstance().getLoginTypeNameParameter();
        String loginTypeConfigValue = CustomDataConfig.getInstance().getLoginTypeValue();
        if (StringKit.isEmpty(loginTypeNameParameter) || StringKit.isEmpty(loginTypeConfigValue)) {
            return false;
        }
        String loginTypeValue = WebUtils.getHTTPRequestParameter(req, loginTypeNameParameter);
        return ComparatorUtils.equals(loginTypeConfigValue, loginTypeValue);
    }


    /**
     * 判断字符串是否全是数字
     *
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        if (StringKit.isEmpty(str)) {
            return false;
        }
        for (int i = str.length(); --i >= 0; ) {
            if (!Character.isDigit(str.charAt(i))) {

                return false;

            }
        }
        return true;
    }


    /**
     * 获取IDM OAuth 用户名
     *
     * @param req
     * @return
     */
    private String getIdmOAuthUsername(HttpServletRequest req) {
        try {
            if (req == null) {
                return "";
            }
            String oAuthCode = WebUtils.getHTTPRequestParameter(req, "ticket");
            if (StringKit.isEmpty(oAuthCode)) {
                return "";
            }
            LogKit.info("集成登录,ticket:" + oAuthCode);
            RequestConfig requestConfig = RequestConfig.custom()
                    .setConnectionRequestTimeout(10000)
                    .setSocketTimeout(10000) // 服务端相应超时
                    .setConnectTimeout(10000) // 建立socket链接超时时间
                    .build();

            //获取Access Token
            //环境地址/getTokens
            String accessTokenUrl = CustomDataConfig.getInstance().getAccessTokenUrl() + "?client_id=" + CustomDataConfig.getInstance().getIdmClientId() + "&client_secret=" + CustomDataConfig.getInstance().getIdmClientSecret() + "&grant_type=authorization_code&code=" + oAuthCode + "&redirect_uri=" + CustomDataConfig.getInstance().getFrUrl();
            LogKit.info("集成登录,获取access_toke url:" + accessTokenUrl);
            HttpGet httpGet = new HttpGet(accessTokenUrl);
            httpGet.addHeader("User-Agent", DEFAULT_USER_AGENT);



            httpGet.setConfig(requestConfig);
            CloseableHttpClient httpClient = Utils.createHttpClient(accessTokenUrl);
            CloseableHttpResponse response = httpClient.execute(httpGet);
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,http status:" + statusCode);
                return "";
            }

            HttpEntity httpEntity = response.getEntity();
            if (httpEntity == null) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,http响应内容为空");
                return "";
            }
            String responseContent = EntityUtils.toString(httpEntity, "UTF-8");
            response.close();
            if (StringKit.isEmpty(responseContent)) {
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,http响应内容为空1");
                return "";
            }
            LogKit.info("集成登录,获取Access Token请求,http响应内容\n" + responseContent);

            String accessToken = getAccessToken(responseContent);
            if (StringKit.isEmpty(accessToken)) {
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,access_token为空");
                return "";
            }
            LogKit.info("集成登录,Access Token:" + accessToken);


            String userUrl = CustomDataConfig.getInstance().getUserUrl() + "?access_token=" + accessToken;
            LogKit.info("集成登录,获取用户信息Url:" + userUrl);
            httpGet = new HttpGet(userUrl);
            httpGet.setConfig(requestConfig);
            httpGet.addHeader("User-Agent", DEFAULT_USER_AGENT);
            response = httpClient.execute(httpGet);
            statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取用户信息请求出错,http status:" + statusCode);
                return "";
            }

            httpEntity = response.getEntity();
            if (httpEntity == null) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取用户信息请求出错,http响应内容为空");
                return "";
            }
            responseContent = EntityUtils.toString(httpEntity, "UTF-8");
            response.close();
            httpClient.close();
            if (StringKit.isEmpty(responseContent)) {
                LogKit.info("集成登录,获取用户信息请求出错,http响应内容为空1");
                return "";
            }
            LogKit.info("集成登录,获取用户信息请求,http响应内容\n" + responseContent);

            String uid = getUsername(responseContent);
            if (StringKit.isEmpty(uid)) {
                LogKit.info("集成登录,获取用户信息请求出错,用户名为空");
                return "";
            }
            LogKit.info("集成登录,用户名:" + uid);
            return uid;
        } catch (Exception e) {
            LogKit.error("集成登录获取用户名出错," + e.getMessage(), e);
        }
        return "";
    }

    private String getAccessToken(String content) {
        if (StringKit.isEmpty(content)) {
            return "";
        }
        JSONObject jsonObject = new JSONObject(content);
        String msg = jsonObject.getString("msg");
        if (!StringKit.equalsIgnoreCase("SUCCESS", msg)) {
            return "";
        }
        String token = jsonObject.getString("access_token");
        return token;
    }

    private String getUsername(String content) {
        if (StringKit.isEmpty(content)) {
            return "";
        }
        JSONObject jsonObject = new JSONObject(content);
        String msg = jsonObject.getString("msg");
        if (!StringKit.equalsIgnoreCase("SUCCESS", msg)) {
            return "";
        }
        String username = jsonObject.getString("id");
        return username;
    }


    private String getRequestUrl(HttpServletRequest req) throws UnsupportedEncodingException {
        String fullUrl = req.getRequestURL().toString();
        Map<String, String[]> paraMap = req.getParameterMap();
        String paraName;
        String[] paraValues;
        String loginTypeParaName = CustomDataConfig.getInstance().getLoginTypeNameParameter();
        String queryStr = "";
        for (Map.Entry<String, String[]> entry : paraMap.entrySet()) {
            paraName = entry.getKey();
            if (ComparatorUtils.equals(paraName, loginTypeParaName)) {
                continue;
            }
            if (ComparatorUtils.equals(paraName, "code")) {
                continue;
            }
            paraValues = entry.getValue();
            LogKit.info("集成登录,获取用户信息请求出错,login_name为空");
            queryStr = addParaToQuery(queryStr, paraName, paraValues);
        }
        if (StringKit.isEmpty(queryStr)) {
            return fullUrl;
        }
        fullUrl = fullUrl + "?" + queryStr;
        return fullUrl;
    }

    private String addParaToQuery(String query, String paraName, String[] paraValues) throws UnsupportedEncodingException {
        if (StringKit.isEmpty(paraName)) {
            return query;
        }
        String fullQuery = query;
        if ((paraValues == null) || (paraValues.length <= 0)) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            fullQuery = paraName + "=";
            return fullQuery;
        }
        String value;
        for (int i = 0, max = paraValues.length - 1; i <= max; i++) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            value = paraValues[i];
            if (StringKit.equals("viewlet", paraName) && (value.indexOf("%") < 0)) {
                value = UriUtils.encodeQueryParam(value, "UTF-8");
            }
            fullQuery = fullQuery + paraName + "=" + value;
        }
        return fullQuery;
    }


}
